@@ -20,16 +20,16 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib/gi18n.h>
-#include <libgnomeui/libgnomeui.h>
 #include "flegitaactionwidget.h"
 #include <cairo.h>
 #include <cairo-pdf.h>
 #include "flegitatypes.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #define FLEGITA_ACTION_WIDGET_ERROR			(g_type_qname (FLEGITA_TYPE_ACTION_WIDGET))
 #define	FLEGITA_ACTION_WIDGET_PARENT_CLASS(klass)	(GTK_WIDGET_CLASS (g_type_class_peek_parent ((klass))))
@@ -55,6 +55,12 @@ struct _FlegitaActionInfo {
   const gchar* 		filename;
   FlegitaFormat		format;
   gint			resolution;
+
+  gboolean		new_image; 	/* wether an incoming image is the
+					   first of an mass acquisition */
+  cairo_t		*cairo;
+  cairo_surface_t	*surface;
+  gchar			*realpath;
 };
 
 enum Column {
@@ -76,10 +82,17 @@ void				faw_action_selected				(GtkComboBox *combo,
 void				faw_format_selected				(GtkComboBox *combo,
 										 FlegitaActionWidget *faw);
 
+void				faw_acquisition_started				(GnomeScanContext* context,
+										 GnomeScanForecast *forecast,
+										 FlegitaActionWidget *faw);
+
 void				faw_image_acquired				(GnomeScanContext *context,
 										 GnomeScanResult *acquisition,
 										 FlegitaActionWidget *faw);
 
+void				faw_acquisition_terminated			(GnomeScanContext *context,
+										 FlegitaActionWidget *faw);
+
 void				faw_dir_selection_changed			(GtkFileChooser *chooser,
 										 FlegitaActionWidget *faw);
 
@@ -308,7 +321,7 @@ flegita_action_widget_new (GnomeScanContext *context)
 
   /* file */
   priv->file = faw_create_file_selector (faw);
-  gtk_box_pack_start (GTK_BOX (widget), priv->file, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (widget), priv->file, TRUE, TRUE, 0);
   gtk_widget_set_no_show_all (priv->file, TRUE);
 
   g_signal_connect (context,
@@ -316,6 +329,16 @@ flegita_action_widget_new (GnomeScanContext *context)
 		    (GCallback) faw_image_acquired,
 		    faw);
 
+  g_signal_connect (context,
+		    "acquisition-started",
+		    (GCallback) faw_acquisition_started,
+		    faw);
+
+  g_signal_connect (context,
+		    "acquisition-terminated",
+		    (GCallback) faw_acquisition_terminated,
+		    faw);
+
   /* preselect */
   gtk_combo_box_set_active (GTK_COMBO_BOX (priv->action), 0);
 
@@ -381,15 +404,26 @@ faw_format_selected (GtkComboBox *combo,
   case FORMAT_PNG:
   case FORMAT_JPEG:
   case FORMAT_TIFF:
-  case FORMAT_PDF:
+/*   case FORMAT_PDF:		/\* For now, we create PDF for screen, not print *\/ */
     info->resolution = gdk_screen_get_resolution (gdk_screen_get_default ());
     break;
-/*     /\* TODO: check correct resolution for PDF (ready to print with nice result) *\/ */
-/*     info->resolution = ; */
-/*     break; */
+  case FORMAT_PDF:
+    /* TODO: check correct resolution for PDF (ready to print with nice result) */
+    info->resolution = 300;
+    break;
   }
 
-  gnome_scan_context_set_resolution (faw->context, info->resolution);
+  gnome_scan_context_set_resolution (faw->context,
+				      info->resolution);
+}
+
+void
+faw_acquisition_started (GnomeScanContext *context,
+			 GnomeScanForecast *forecast,
+			 FlegitaActionWidget *faw)
+{
+  FlegitaActionWidgetPrivate *priv = GET_PRIVATE (faw);
+  priv->info->new_image = TRUE;
 }
 
 void
@@ -406,6 +440,24 @@ faw_image_acquired (GnomeScanContext *context,
   }
 }
 
+void
+faw_acquisition_terminated (GnomeScanContext *context,
+			    FlegitaActionWidget *faw)
+{
+  FlegitaActionWidgetPrivate *priv = GET_PRIVATE (faw);
+
+  if (priv->info->cairo) {
+    cairo_destroy (priv->info->cairo);
+    priv->info->cairo = NULL;
+    cairo_surface_destroy (priv->info->surface);
+    priv->info->surface = NULL;
+    g_free (priv->info->realpath);
+    priv->info->realpath = NULL;
+  }
+}
+
+
+
 /********************************
  * 	      OTHER		*
  ********************************/
@@ -460,15 +512,7 @@ faw_create_contact_selector (FlegitaActionWidget *faw)
 
   box = gtk_hbox_new (FALSE, 6);
 
-  /* translator, label "to" before [contact] field */
-  widget = gtk_label_new_with_mnemonic (_("to"));
-  gtk_box_pack_start (GTK_BOX (box), widget, FALSE, TRUE, 0);
-
-  widget = gtk_entry_new ();
-  gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
-  gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
-
-  /* TODO: contact completion */
+  /* TODO: Choose attached filename and format  */
 
   return box;
 }
@@ -477,7 +521,6 @@ GtkWidget*
 faw_create_dir_selector (FlegitaActionWidget *faw)
 {
   GtkWidget *box, *widget;
-  gchar *dir;
 
   box = gtk_hbox_new (FALSE, 6);
   /* translators, label "in" before [dir] selector */
@@ -495,9 +538,7 @@ faw_create_dir_selector (FlegitaActionWidget *faw)
 
   /* default value */
   /* TODO: store last dir */
-  dir = g_build_filename (g_get_home_dir (), "Documents", NULL);
-  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (widget), dir);
-  g_free (dir);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (widget), g_get_home_dir ());
 
   return box;
 }
@@ -555,7 +596,7 @@ faw_create_file_selector (FlegitaActionWidget *faw)
   		    (GCallback) faw_format_selected,
   		    faw);
 
-  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), FORMAT_PNG);
 
   gtk_box_pack_start (GTK_BOX (box), widget, FALSE, TRUE, 0);
 
@@ -601,6 +642,7 @@ faw_action_save (GnomeScanResult *acquisition,
 
     if (filename)
       g_free (filename);
+
     filename = g_build_filename (info->dir, basename, NULL);
 
     n++;
@@ -611,13 +653,13 @@ faw_action_save (GnomeScanResult *acquisition,
   case FORMAT_JPEG:
   case FORMAT_TIFF:
     faw_write_image (acquisition->image, filename, info->format);
+    g_free (filename);
     break;
   case FORMAT_PDF:
-    faw_write_pdf (info, acquisition->image, filename);
+    info->realpath = info->realpath ? info->realpath : filename;
+    faw_write_pdf (info, acquisition->image, info->realpath);
     break;
   }
-
-  g_free (filename);
 }
 
 /* maybe the simplest backend for so many format :) */
@@ -638,12 +680,24 @@ faw_write_pdf (FlegitaActionInfo *info,
   cairo_t *cr;
   cairo_surface_t *cs;
 
-  cs = cairo_pdf_surface_create (filename, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf));
-  cairo_surface_set_fallback_resolution (cs,(gint) info->resolution, (gint) info->resolution);
-  cr = cairo_create (cs);
+  g_debug ("Writing to %s", info->realpath);
+  if (info->new_image) {
+    info->surface = cs = cairo_pdf_surface_create (info->realpath,
+						   /* dpi / 72 = dot per point */
+						   gdk_pixbuf_get_width (pixbuf) / (info->resolution / 72),
+						   gdk_pixbuf_get_height (pixbuf) / (info->resolution / 72));
+    cairo_surface_set_fallback_resolution (cs,(gint) info->resolution, (gint) info->resolution);
+    info->cairo = cr = cairo_create (cs);
+    info->new_image = FALSE;
+  }
+  else {
+    cr = info->cairo;
+    cs = info->surface;
+  }
+
+  /* paint the image in a page */
   gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
   cairo_paint (cr);
   cairo_show_page (cr);
-  cairo_destroy (cr);
-  cairo_surface_destroy (cs);
+/*   cairo_surface_flush (cs); */
 }
