@@ -31,11 +31,11 @@
  */
 
 #include <string.h>
-#include "gnomescan-intl.h"
 #define	SANE_I18N(text)	_(text)
 #include <sane/sane.h>
 #include <sane/saneopts.h>
 #include "gnomescan.h"
+#include "gnomescan-intl.h"
 
 #define GNOME_SCAN_BACKEND_ERROR		(gnome_scan_backend_error_quark ())
 #define	GNOME_SCAN_BACKEND_PARENT_CLASS(klass)	(G_OBJECT_CLASS (g_type_class_peek_parent ((klass))))
@@ -70,6 +70,7 @@ struct _GnomeScannerId {
   GSList*		buttons;
   /* sources flags correspondance table */
   GHashTable		*sources;
+  GHashTable		*colorspaces;
 };
 
 struct _GnomeScannerOptionId {
@@ -99,6 +100,8 @@ void				gsb_scanner_destroy			(GnomeScannerId *sid);
 GnomeScannerType		gsb_scanner_get_type			(GnomeScannerId *sid);
 GnomeScanGeometry*		gsb_scanner_get_geometry		(GnomeScannerId *sid);
 GValueArray*			gsb_scanner_get_sources			(GnomeScannerId *sid);
+GValueArray*			gsb_scanner_get_colorspaces		(GnomeScannerId *sid);
+GValueArray*			gsb_scanner_get_depths			(GnomeScannerId *sid);
 
 GHashTable*			gsb_scanner_get_options			(GnomeScannerId *sid,
 									 GError **error);
@@ -117,6 +120,20 @@ void				gsb_scanner_source_add			(GnomeScannerSource *value,
 									 const gchar *source,
 									 GValueArray *array);
 
+/* scan colorspace function */
+void				gsb_scanner_colorspace_parse		(GValue *value,
+									 GnomeScannerId *sid);
+
+void				gsb_scanner_colorspace_add		(GnomeScanColorspace *value,
+									 const gchar *colorspace,
+									 GValueArray *array);
+
+void				gsb_scanner_depth_add			(GValue *value,
+									 GValueArray *array);
+
+void                            gsb_scanner_resolution_add              (GValue *value,
+                                                                         GValueArray *array);
+
 /* (real) Options functions (either scan or device options) */
 GValue*				gsb_scanner_option_get_value		(GnomeScannerId *sid,
 									 GnomeScannerOptionId *oid,
@@ -136,7 +153,6 @@ void				gsb_scanner_option_reload		(gconstpointer key,
 									 GnomeScannerOption *option,
 									 GnomeScannerId *sid);
 
-/* Acquisition functions */
 GnomeScanForecast*		gsb_scan_forecast			(GnomeScanContextInfo *info,
 									 GError **error);
 
@@ -149,9 +165,52 @@ void				gsb_scan_apply_option			(GnomeScanContextInfo *info,
 									 GError **error);
 
 
+GValue*				gsb_scan_apply_option_value_resolution	(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
+GValue*				gsb_scan_apply_option_value_area	(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
+GValue*				gsb_scan_apply_option_value_preview	(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
+GValue*				gsb_scan_apply_option_value_source	(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
+GValue*				gsb_scan_apply_option_value_colorspace	(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
+GValue*				gsb_scan_apply_option_value_depth	(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
+GValue*				gsb_scan_apply_option_value_test_picture(GnomeScannerId *sid,
+									 GnomeScannerOptionId *oid,
+									 GnomeScanContextInfo *info);
+
 void				gsb_scan_apply_context			(GnomeScanContextInfo *info,
 									 GError **error);
 
+/* Acquisition functions */
+
+GnomeScanResult*		gsb_acquire_one				(GnomeScanBackend *backend,
+									 GnomeScannerId *sid,
+									 GnomeScanContextInfo *info,
+									 GError **error);
+
+void				gsb_convert_data_to_rgb			(SANE_Frame format,
+									 gint depth,
+									 guchar *data,
+									 guchar *pixels,
+									 gint offset,
+									 gint length);
+
+
 /* misc */
 GError*				gsb_get_error				(SANE_Status status);
 
@@ -161,7 +220,8 @@ GError*				gsb_get_error				(SANE_Status status);
 
 
 enum {
-  SIGNAL_DATA_RECEIVED,
+  DATA_RECEIVED,
+  IMAGE_ACQUIRED,
   N_SIGNALS
 };
 
@@ -171,7 +231,7 @@ static guint signals[N_SIGNALS];
 
 G_DEFINE_TYPE (GnomeScanBackend, gnome_scan_backend, G_TYPE_OBJECT)
 
-void
+     void
 gnome_scan_backend_class_init (GnomeScanBackendClass *klass)
 {
   GObjectClass* gobject_class = G_OBJECT_CLASS (klass);
@@ -192,16 +252,34 @@ gnome_scan_backend_class_init (GnomeScanBackendClass *klass)
    * Emitted when the backend received data from the scanner during an
    * acquisition.
    **/
-  signals[SIGNAL_DATA_RECEIVED] = g_signal_new ("data-received",
-						GNOME_TYPE_SCAN_BACKEND,
-						G_SIGNAL_RUN_FIRST,
-						G_STRUCT_OFFSET (GnomeScanBackendClass, data_received),
-						NULL,
-						NULL,
-						g_cclosure_marshal_VOID__INT,
-						G_TYPE_NONE,
-						1,
-						G_TYPE_INT);
+  signals[DATA_RECEIVED] = g_signal_new ("data-received",
+					 GNOME_TYPE_SCAN_BACKEND,
+					 G_SIGNAL_RUN_FIRST,
+					 G_STRUCT_OFFSET (GnomeScanBackendClass, data_received),
+					 NULL,
+					 NULL,
+					 g_cclosure_marshal_VOID__INT,
+					 G_TYPE_NONE,
+					 1,
+					 G_TYPE_INT);
+
+  /**
+   * GnomeScanBackend::image-acquired:
+   * @backend: the emitting #GnomeScanBackend
+   * @result: the scan result containing the acquired image
+   *
+   * Emitted when the backend successfully acquired an image.
+   **/
+  signals[IMAGE_ACQUIRED] = g_signal_new ("image-acquired",
+					  GNOME_TYPE_SCAN_BACKEND,
+					  G_SIGNAL_RUN_FIRST,
+					  G_STRUCT_OFFSET (GnomeScanBackendClass, image_acquired),
+					  NULL,
+					  NULL,
+					  g_cclosure_marshal_VOID__BOXED,
+					  G_TYPE_NONE,
+					  1,
+					  GNOME_TYPE_SCAN_RESULT);
 }
 
 void
@@ -224,6 +302,7 @@ gnome_scan_backend_error_quark (void)
 
 /**
  * gnome_scan_backend_new:
+ * @error:	A #GError
  * 
  * Instanciate a new backend. This function is also responsible to
  * init the third party backend (e.g. SANE).
@@ -236,6 +315,8 @@ gnome_scan_backend_new (GError **error)
   SANE_Status status;
   gint version;
 
+  g_debug ("%s %s", PACKAGE, VERSION);
+
   /* ONLY SUPPORT SANE 1 */
   if (SANE_CURRENT_MAJOR != 1) {
     *error = g_error_new (GNOME_SCAN_BACKEND_ERROR,
@@ -282,7 +363,8 @@ gnome_scan_backend_destroy (GnomeScanBackend *backend)
 
 /**
  * gnome_scan_backend_probe_scanners:
- * @backend: a #GnomeScanBackend
+ * @backend:	a #GnomeScanBackend
+ * @error:	a #GError
  * 
  * Do the long device probe task and return a #GSList of
  * #GnomeScanner.
@@ -300,7 +382,12 @@ gnome_scan_backend_probe_scanners (GnomeScanBackend *backend,
   GSList *scanners;
   gint i;
 
-#if	ENABLE_DEBUGGING
+#if	ENABLE_PROBE
+  const SANE_Device **devices;
+  g_debug ("sane_get_devices (%p, FALSE)", &devices);
+  status = sane_get_devices (&devices, FALSE);
+#else
+  g_debug ("Fake device probe. Using 'test' SANE device.");
   SANE_Device **devices;
   devices = g_new0(SANE_Device*, 2);
   devices[0] = g_new0(SANE_Device, 1);
@@ -309,10 +396,6 @@ gnome_scan_backend_probe_scanners (GnomeScanBackend *backend,
   devices[0]->model = "Test device";
   devices[0]->type = "virtual device";
   status = SANE_STATUS_GOOD;
-#else
-  const SANE_Device **devices;
-  g_debug ("sane_get_devices (%p, FALSE)", &devices);
-  status = sane_get_devices (&devices, FALSE);
 #endif
 
   scanners = NULL;
@@ -369,26 +452,18 @@ gnome_scan_backend_forecast (GnomeScanBackend *backend,
  * device sent data. You should use
  * gnome_scan_context_start_acquisition().
  * 
- * Return value: the acquisition result
  **/
-GnomeScanResult*
+void
 gnome_scan_backend_acquire (GnomeScanBackend *backend,
 			    GnomeScanContextInfo *info,
 			    GError **error)
 {
-  g_return_val_if_fail (GNOME_IS_SCAN_BACKEND (backend), NULL);
-  g_return_val_if_fail (GNOME_IS_SCANNER (info->scanner), NULL);
+  g_return_if_fail (GNOME_IS_SCAN_BACKEND (backend));
+  g_return_if_fail (GNOME_IS_SCANNER (info->scanner));
 
   SANE_Status status;
   GnomeScannerId *sid;
-  GnomeScannerOption *option;
-  GValue *value;
-  GnomeScanForecast *fc;
   GnomeScanResult *result;
-  guchar *buffer;
-  gint count, len;
-
-  result = NULL;
 
   gsb_scan_apply_context (info, error);
   if (*error) {
@@ -396,89 +471,28 @@ gnome_scan_backend_acquire (GnomeScanBackend *backend,
   }
   else {
     sid = gnome_scanner_get_id (info->scanner);
-    status =  sane_start (sid->handle);
-    *error = gsb_get_error (status);
-
-    if (*error) {
-      /* we should handle properly INVAL and other status like that */
-      g_debug ("Failed to start scan from %s: %s", sid->descriptor->name, (*error)->message);
-    }
-    else {
-      fc = gsb_scan_forecast (info, error);
-
-      if (*error) {
-	g_debug ("Failed to forecast acquisition !");
+    do {
+      result = gsb_acquire_one (backend, sid, info, error);
+      if (result) {
+	g_signal_emit_by_name (backend, "image-acquired", result);
+	g_boxed_free (GNOME_TYPE_SCAN_RESULT, result);
       }
-      else {
-	g_debug ("Scan %f×%f (%iKB)", fc->geometry.width, fc->geometry.height, fc->size);
-	buffer = g_malloc0 (fc->size);
-	count = 0;
-
-	do {
-	  status = sane_read (sid->handle, buffer + count, fc->size - count, &len);
-
-	  if (status == SANE_STATUS_GOOD) {
-	    count+= len;
-	    g_signal_emit_by_name (backend, "data-received", count);
-	    g_debug ("Receveid %iB. %f%%", len, 100. * (gdouble) count / (gdouble) fc->size);
-	  }
-	} while (status == SANE_STATUS_GOOD);
-
-	if (status == SANE_STATUS_EOF) {
-	  /* store file */
-	  result = g_new0 (GnomeScanResult, 1);
-
-	  /* we retreive the actual resolution */
-	  option = g_hash_table_lookup (sid->hidden, "resolution");
-	  if (option) {
-	    value = gsb_scanner_option_get_value (sid, gnome_scanner_option_get_id (option), error);
-	    switch (G_VALUE_TYPE (value)) {
-	    case G_TYPE_INT:
-	      result->resolution = (gdouble) g_value_get_int (value);
-	      break;
-	    case G_TYPE_DOUBLE:
-	      result->resolution = g_value_get_double (value);
-	      break;
-	    }
-	  }
-	  /* or use the asked one. */
-	  else {
-	    result->resolution = info->resolution;
-	  }
-
-	  result->image = gdk_pixbuf_new_from_data (buffer, GDK_COLORSPACE_RGB, FALSE,
-						    8, fc->geometry.width, fc->geometry.height, fc->rowstride,
-						    (GdkPixbufDestroyNotify) g_free, NULL);
-	  g_debug ("Sent picture %p", result->image);}
-	    
-	  
-	else if (status != SANE_STATUS_CANCELLED) {
-	  /* other errors are jammed, no docs, cover open, etc. */
-	  *error = gsb_get_error (status);
-	}
-      }
-    }
+    } while (!info->preview &&
+	     info->source == GNOME_SCANNER_SOURCE_AUTOMATIC_DOCUMENT_FEEDER
+	     && result);
   }
-
-  /*
-   * Some backend NEED to cancel an acquisition even if terminated
-   * (e.g. hpaio).
-   */
-  sane_cancel (sid->handle);
-  
-  return result;
 }
 
 /**
- * gnome_scan_backend_cancel:
+ * gnome_scan_backend_stop:
  * @backend: a #GnomeScanBackend
  * 
  * Cancel acquisition (preview or final). You should use
- * gnome_scan_context_cancel_acquisition() instead.
+ * gnome_scan_context_stop_acquisition() instead.
  **/
 void
-gnome_scan_backend_cancel (GnomeScanBackend *backend,
-			   GnomeScanner *scanner)
+gnome_scan_backend_stop (GnomeScanBackend *backend,
+			 GnomeScanner *scanner)
 {
   g_return_if_fail (GNOME_IS_SCAN_BACKEND (backend));
   GnomeScannerId *sid = gnome_scanner_get_id (scanner);
@@ -507,9 +521,11 @@ gsb_scanner_new (GnomeScanBackend *backend,
   gchar *vendor,  *product;
   GnomeScannerType type;
   GnomeScannerOption *option;
-  GValueArray *sources;
+  GValueArray *sources, *colorspaces, *depths;
   GnomeScanGeometry *geometry;
+  GnomeScannerOptionConstraint *constraint;
   GnomeScanRange *resolution_range;
+  GValueArray *resolution_enum;
 
   /*
    * Now we go further sane, we probe all options and distribute them
@@ -554,8 +570,30 @@ gsb_scanner_new (GnomeScanBackend *backend,
       /* search for buttons. */
       g_hash_table_foreach_steal (sid->options, (GHRFunc) gsb_scanner_steal_button, sid);
 
-      sources = gsb_scanner_get_sources (sid);
-      geometry = gsb_scanner_get_geometry (sid);
+      option = g_hash_table_lookup (sid->hidden, "resolution");
+      if (option) {
+        constraint = gnome_scanner_option_get_constraint (option);
+
+      	if (gnome_scanner_option_get_flags(option) & GNOME_SCANNER_OPTION_RANGE) {
+	  resolution_range = constraint->range;
+        }
+	else {
+	resolution_range = NULL;
+	}
+
+        if (gnome_scanner_option_get_flags(option) & GNOME_SCANNER_OPTION_ENUMERATION) {
+	  resolution_enum = g_value_array_new (g_slist_length (constraint->enumeration));
+	  g_slist_foreach (constraint->enumeration, (GFunc) gsb_scanner_resolution_add, resolution_enum);
+        }
+	else {
+	resolution_enum = NULL;
+	}
+      }
+
+      sources 		= gsb_scanner_get_sources (sid);
+      colorspaces	= gsb_scanner_get_colorspaces (sid);
+      depths		= gsb_scanner_get_depths (sid);
+      geometry  	= gsb_scanner_get_geometry (sid);
       scanner = g_object_new (GNOME_TYPE_SCANNER,
 			      "id", sid,
 			      "vendor", vendor,
@@ -563,7 +601,11 @@ gsb_scanner_new (GnomeScanBackend *backend,
 			      "type", type,
 			      "geometry", geometry,
 			      "sources", sources,
+			      "colorspaces", colorspaces,
+			      "depths", depths,
 			      "options", sid->options,
+			      "resolution-range", resolution_range,
+			      "resolution-enum", resolution_enum,
 			      NULL);
     }
   }
@@ -696,13 +738,13 @@ gsb_scanner_get_geometry (GnomeScannerId *sid)
 
   geometry = g_new0 (GnomeScanGeometry, 1);
 
-  option = g_hash_table_lookup (sid->hidden, "tl-x");
+  option = g_hash_table_lookup (sid->hidden, "br-x");
   constraint = gnome_scanner_option_get_constraint (option);
-  geometry->width = constraint->range.upper;
+  geometry->width = constraint->range->upper;
 
-  option = g_hash_table_lookup (sid->hidden, "tl-y");
+  option = g_hash_table_lookup (sid->hidden, "br-y");
   constraint = gnome_scanner_option_get_constraint (option);
-  geometry->height = constraint->range.upper;
+  geometry->height = constraint->range->upper;
 
   return geometry;
 }
@@ -740,6 +782,57 @@ gsb_scanner_get_sources (GnomeScannerId *sid)
   return result;
 }
 
+/**
+ * Parse mode in return each mode in an array of colorspace.
+ */
+GValueArray*
+gsb_scanner_get_colorspaces (GnomeScannerId *sid)
+{
+  GValueArray* result;
+  GnomeScannerOption *option;
+  GnomeScannerOptionConstraint *constraint;
+
+  option = g_hash_table_lookup (sid->hidden, "mode");
+  if (option) {
+    sid->colorspaces = g_hash_table_new (g_int_hash, g_int_equal);
+    constraint = gnome_scanner_option_get_constraint (option);
+    g_slist_foreach (constraint->enumeration, (GFunc) gsb_scanner_colorspace_parse, sid);
+
+    result = g_value_array_new (g_hash_table_size (sid->colorspaces));
+    g_hash_table_foreach (sid->colorspaces, (GHFunc) gsb_scanner_colorspace_add, result);
+  }
+  else {
+    g_debug ("Emulating option 'mode' allowing only RGB.");
+    result = g_value_array_new (1);
+    g_value_array_append (result, gs_value_new_enum (GNOME_TYPE_SCAN_COLORSPACE, GNOME_SCAN_COLORSPACE_RGB));
+  }
+
+  return result;
+}
+
+GValueArray*
+gsb_scanner_get_depths (GnomeScannerId *sid)
+{
+  GValueArray *result;
+  GnomeScannerOption *option;
+  GnomeScannerOptionConstraint *constraint;
+
+
+  option = g_hash_table_lookup (sid->hidden, "depth");
+  if (option) {
+    constraint = gnome_scanner_option_get_constraint (option);
+    result = g_value_array_new (g_slist_length (constraint->enumeration));
+    g_slist_foreach (constraint->enumeration, (GFunc) gsb_scanner_depth_add, result);
+  }
+  else {
+    g_debug ("Emulating option 'depth'");
+    result = g_value_array_new (1);
+    g_value_array_append (result, gs_value_new_int (8));
+  }
+
+  return result;
+}
+
 /*
  * In SANE 1, buttons are option with button in the name. We separate
  * button from other options.
@@ -783,7 +876,7 @@ gsb_scanner_source_parse (GValue *value,
    * Note that this is incomplete. Have to read all backend code in
    * order to know which string they use.
    *
-   * Tested only with plustek, hpoj and hpaio backend.
+   * Tested only with test, plustek, hpoj and hpaio backend.
    */
   if (gs_strcmp (str, "Flatbed") == 0) {
     *source = GNOME_SCANNER_SOURCE_FLATBED;
@@ -817,7 +910,69 @@ gsb_scanner_source_add (GnomeScannerSource *value,
   g_value_array_append (array, gs_value_new_enum (GNOME_TYPE_SCANNER_SOURCE, *value));
 }
 
+void
+gsb_scanner_depth_add (GValue *value,
+		       GValueArray *array)
+{
+  g_value_array_append (array, gs_value_new_int (g_value_get_int (value)));
+}
 
+void
+gsb_scanner_resolution_add (GValue *value,
+                       GValueArray *array)
+{
+  GValue *vdouble = gs_value_new_double (0.);
+  g_value_transform (value, vdouble);
+  g_value_array_append (array, vdouble);
+}
+
+
+/* SCAN COLORSPACES FUNCTIONS */
+
+/*
+ * Parse a colorspace and keep string <=> flag correspondance in a
+ * sid->colorspaces hash table.
+ */
+void
+gsb_scanner_colorspace_parse (GValue *value,
+			      GnomeScannerId *sid)
+{
+  const gchar *str = g_value_get_string (value);
+  gint *colorspace = g_new0 (gint, 1);
+
+  /* all tests :(
+   *
+   * Note that this is incomplete. Have to read all backend code in
+   * order to know which string they use.
+   *
+   * Tested only with plustek, hpoj and hpaio backend.
+   */
+  if (gs_strcmp (str, "Gray") == 0) {
+    *colorspace = GNOME_SCAN_COLORSPACE_GRAY;
+  }
+  else if (gs_strcmp (str, "Lineart") == 0) {
+    *colorspace = GNOME_SCAN_COLORSPACE_LINEART;
+  }
+  else if (gs_strcmp (str, "Halftone") == 0) {
+    *colorspace = GNOME_SCAN_COLORSPACE_HALFTONE;
+  }
+  else {
+    *colorspace = GNOME_SCAN_COLORSPACE_RGB;
+  }
+
+  g_hash_table_insert (sid->colorspaces, colorspace, g_strdup (str));
+}
+
+/*
+ * Add colorspaces to GValueArray
+ */
+void
+gsb_scanner_colorspace_add (GnomeScanColorspace *value,
+			    const gchar *colorspace,
+			    GValueArray *array)
+{
+  g_value_array_append (array, gs_value_new_enum (GNOME_TYPE_SCAN_COLORSPACE, *value));
+}
 
 /* SCANNER OPTION FUNCTIONS */
 
@@ -834,7 +989,6 @@ gsb_scanner_option_get_value (GnomeScannerId *sid,
   GValue *value;
   GType type;
   gpointer data;
-  gchar *string = " value\t= ";
 
   value = g_new0 (GValue, 1);
   type = gsb_scanner_option_get_type (oid);
@@ -849,24 +1003,20 @@ gsb_scanner_option_get_value (GnomeScannerId *sid,
   switch (type) {
   case G_TYPE_BOOLEAN:
     g_value_set_boolean (value, *((gboolean*) data));
-    string = g_strdup_printf ("%s%s", string, (g_value_get_boolean (value) ? "TRUE" : "FALSE"));
     break;
   case G_TYPE_INT:
     g_value_set_int (value, *((gint*) data));
-    string = g_strdup_printf ("%s%i", string, g_value_get_int (value));
     break;
   case G_TYPE_DOUBLE:
     g_value_set_double (value, SANE_UNFIX (*((gint*) data)));
-    string = g_strdup_printf ("%s%f", string, g_value_get_double (value));
     break;
   case G_TYPE_STRING:
     g_value_set_string (value, data);
-    string = g_strdup_printf ("%s%s", string, g_value_get_string (value));
     break;
   default:
     break;
   }
-  g_debug (string);
+  g_debug (" value\t= %s", g_strdup_value_contents (value));
 
   return value;
 }
@@ -885,10 +1035,11 @@ gsb_scanner_option_set_value (GnomeScannerId *sid,
   gpointer data;
   gboolean boolean;
   gint info, action, integer;
-  gchar *string = g_strdup_printf ("Setting option %s %s = ", sid->descriptor->name, oid->descriptor->name);
 
-  if (oid->descriptor->cap & SANE_CAP_INACTIVE)
+  if (oid->descriptor->cap & SANE_CAP_INACTIVE) {
+    g_debug ("Not setting inactive option %s %s", sid->descriptor->name, oid->descriptor->name);
     return;
+  }
 
   if (value) {
     action = SANE_ACTION_SET_VALUE;
@@ -897,39 +1048,37 @@ gsb_scanner_option_set_value (GnomeScannerId *sid,
     case G_TYPE_INT:
       integer = g_value_get_int (value);
       data = &integer;
-      string = g_strdup_printf ("%s%i", string, integer);
       break;
     case G_TYPE_DOUBLE:
       integer = SANE_FIX (g_value_get_double (value));
       data = &integer;
-      string = g_strdup_printf ("%s%f(%x)", string, g_value_get_double (value), integer);
       break;
     case G_TYPE_BOOLEAN:
       boolean = g_value_get_boolean (value);
       data = &boolean;
-      string = g_strdup_printf ("%s%s", string, (g_value_get_boolean (value) ? "TRUE" : "FALSE"));
       break;
     case G_TYPE_STRING:
       data = g_strdup (g_value_get_string (value));
-      string = g_strdup_printf ("%s%s", string, data);
       break;
     default:
       break;
     }
   }
   else if (oid->descriptor->cap & SANE_CAP_HARD_SELECT) {
+    g_debug ("Auto set option value for %s %s", sid->descriptor->name, oid->descriptor->name);
     data = NULL;
     action = SANE_ACTION_SET_AUTO;
   }
   else {
+    g_debug ("Not setting option %s %s !", sid->descriptor->name, oid->descriptor->name);
     *error = g_error_new (GNOME_SCAN_BACKEND_ERROR,
 			  GNOME_SCAN_BACKEND_ERROR_FAILED,
-			  _("Option %s of device %s is not auto settable, you must provide a non-NULL value."),
+			  "Option %s of device %s is not auto settable, you must provide a non-NULL value.",
 			  sid->descriptor->name, oid->descriptor->name);
     return;
   }
 
-  g_debug (string);
+  g_debug ("Setting option %s %s = %s", sid->descriptor->name, oid->descriptor->name, g_strdup_value_contents (value));
   status = sane_control_option (sid->handle, oid->id, action, data, &info);
   *error = gsb_get_error (status);
 
@@ -1032,21 +1181,22 @@ gsb_scanner_option_get_constraint (GnomeScannerOptionId *oid)
   switch (oid->descriptor->constraint_type) {
   case SANE_CONSTRAINT_RANGE:
     /* values belong to {x = lower + k * step; k € N; x < upper} */
+    constraint->range = g_new0 (GnomeScanRange, 1);
     switch (oid->descriptor->type) {
     case SANE_TYPE_INT:
-      constraint->range.lower = (gdouble) oid->descriptor->constraint.range->min;
-      constraint->range.upper = (gdouble) oid->descriptor->constraint.range->max;
-      constraint->range.step = (gdouble) oid->descriptor->constraint.range->quant;
+      constraint->range->lower = (gdouble) oid->descriptor->constraint.range->min;
+      constraint->range->upper = (gdouble) oid->descriptor->constraint.range->max;
+      constraint->range->step = (gdouble) oid->descriptor->constraint.range->quant;
       break;
     case SANE_TYPE_FIXED:
-      constraint->range.lower = SANE_UNFIX (oid->descriptor->constraint.range->min);
-      constraint->range.upper = SANE_UNFIX (oid->descriptor->constraint.range->max);
-      constraint->range.step = SANE_UNFIX (oid->descriptor->constraint.range->quant);
+      constraint->range->lower = SANE_UNFIX (oid->descriptor->constraint.range->min);
+      constraint->range->upper = SANE_UNFIX (oid->descriptor->constraint.range->max);
+      constraint->range->step = SANE_UNFIX (oid->descriptor->constraint.range->quant);
       break;
     default:
       break;
     }
-    g_debug (" range\t= [%f ; %f] step = %f", constraint->range.lower, constraint->range.upper, constraint->range.step);
+    g_debug (" range\t= [%f ; %f] step = %f", constraint->range->lower, constraint->range->upper, constraint->range->step);
     break;
     /* We use GValue so we treat int, fixed or string the same way. */
   case SANE_CONSTRAINT_WORD_LIST:
@@ -1176,22 +1326,39 @@ gsb_scan_apply_option (GnomeScanContextInfo *info,
   GnomeScannerOption *option;
   GnomeScannerOptionId *oid;
   GValue *value = NULL;
-  GType type;
 
-  g_debug ("Apply '%s' option", oname);
   option = g_hash_table_lookup (sid->hidden, oname);
 
   if (option) {
     oid = gnome_scanner_option_get_id (option);
     value = (*value_func) (sid, oid, info);
-    gsb_scanner_option_set_value, (sid, oid, value, error);
+    gsb_scanner_option_set_value (sid, oid, value, error);
     g_free (value);
   }
   else {
-    g_debug ("Option %s is unknowned for device %s", oname, sid->descriptor->name);
+    g_debug ("Option %s is unknown for device %s. Skipping.", oname, sid->descriptor->name);
   }
 }
 
+void
+gsb_scan_apply_test_option (GnomeScannerId *sid,
+			    const gchar* oname,
+			    GValue *value,
+			    GError **error)
+{
+  GnomeScannerOption *option;
+  GnomeScannerOptionId *oid;
+
+  option = g_hash_table_lookup (sid->options, oname);
+
+  if (option) {
+    oid = gnome_scanner_option_get_id (option);
+    gsb_scanner_option_set_value (sid, oid, value, error);
+  }
+  else {
+    g_debug ("Option %s is unknown for device %s. Skipping.", oname, sid->descriptor->name);
+  }
+}
 
 /* Return a GValue settable to SANE option value */
 GValue*
@@ -1200,27 +1367,98 @@ gsb_scan_apply_option_value_resolution (GnomeScannerId *sid,
 					GnomeScanContextInfo *info)
 {
   GValue *value;
-  GType type = gsb_scanner_option_get_type (oid);
+  GnomeScannerOption *option = g_hash_table_lookup (sid->hidden, oid->descriptor->name);
+  GType type = gnome_scanner_option_get_value_type (option);
 
   switch (type) {
-    case G_TYPE_INT:
-      value = gs_value_new_int ((gint) info->resolution);
-       g_debug ("\tresolution = %i dpi", g_value_get_int (value));
-      break;
-    case G_TYPE_DOUBLE:
-      value = gs_value_new_double (info->resolution);
-       g_debug ("\tresolution = %f dpi", g_value_get_double (value));
-      break;
+  case G_TYPE_INT:
+    value = gs_value_new_int ((gint) info->resolution);
+    break;
+  case G_TYPE_DOUBLE:
+    value = gs_value_new_double (info->resolution);
+    break;
   default:
-    g_debug ("Option resolution of type %s", g_type_name (type));
+    g_debug ("Option resolution of type %s not supported.", g_type_name (type));
     break;
   }
 
   return value;
 }
 
+GValue*
+gsb_scan_apply_option_value_area (GnomeScannerId *sid,
+				  GnomeScannerOptionId *oid,
+				  GnomeScanContextInfo *info)
+{
+  /* TODO: handle int {tl,br}-{x,y} */
+  /* TODO: handle pixel {tl,br}-{x,y} */
+  GValue *value;
+  const gchar* oname = oid->descriptor->name;
+
+  value = g_new0 (GValue, 1);
+  g_value_init (value, G_TYPE_DOUBLE);
+
+  switch (oname[0]) {
+  case 't':
+    switch (oname[3]) {
+    case 'x':
+      g_value_set_double (value, info->area->x);
+      break;
+    case 'y':
+      g_value_set_double (value, info->area->y);
+      break;
+    }
+    break;
+  case 'b':
+    switch (oname[3]) {
+    case 'x':
+      g_value_set_double (value, info->area->x + info->area->width);
+      break;
+    case 'y':
+      g_value_set_double (value, info->area->y + info->area->height);
+      break;
+    }
+    break;
+  }
+
+  return value;
+}
+
+GValue*
+gsb_scan_apply_option_value_preview (GnomeScannerId *sid,
+				     GnomeScannerOptionId *oid,
+				     GnomeScanContextInfo *info)
+{
+  return gs_value_new_boolean (info->preview);
+}
+
+GValue*
+gsb_scan_apply_option_value_source (GnomeScannerId *sid,
+				    GnomeScannerOptionId *oid,
+				    GnomeScanContextInfo *info)
+{
+  return gs_value_new_string (g_hash_table_lookup (sid->sources, g_memdup (&(info->source), sizeof (GnomeScannerSource))));
+}
+
+GValue*
+gsb_scan_apply_option_value_colorspace	(GnomeScannerId *sid,
+					 GnomeScannerOptionId *oid,
+					 GnomeScanContextInfo *info)
+{
+  gchar* value = g_hash_table_lookup (sid->colorspaces, g_memdup (&(info->colorspace), sizeof (GnomeScanColorspace)));
+  return value ? gs_value_new_string (value) : NULL;
+}
+
+GValue*
+gsb_scan_apply_option_value_depth	(GnomeScannerId *sid,
+					 GnomeScannerOptionId *oid,
+					 GnomeScanContextInfo *info)
+{
+  return gs_value_new_int (info->depth);
+}
+
 /*
- * Apply scan option to sane.
+ * Apply scan option to sane device.
  */
 void
 gsb_scan_apply_context (GnomeScanContextInfo *info,
@@ -1232,54 +1470,22 @@ gsb_scan_apply_context (GnomeScanContextInfo *info,
   GValue *value;
 
   sid = gnome_scanner_get_id (info->scanner);
-  g_debug ("Applying scan option to %s", sid->descriptor->name);
-
-#define gsb_scanner_get_option_id(s,o)	(option = g_hash_table_lookup (s->hidden, o)) ? gnome_scanner_option_get_id (option) : NULL
+  g_debug ("Applying scan options to %s", sid->descriptor->name);
 
   gsb_scan_apply_option (info, sid, "resolution", (GSBOptionValueFunc) gsb_scan_apply_option_value_resolution, error);
-
-  oid = gsb_scanner_get_option_id (sid, "preview");
-  if (oid) {
-    value = gs_value_new_boolean (info->preview);
-    gsb_scanner_option_set_value (sid, oid, value, error);
-    g_free (value);
-  }
-
-  /* TODO: handle int {tl,br}-{x,y} */
-  /* TODO: handle pixel {tl,br}-{x,y} */
-  oid = gsb_scanner_get_option_id (sid, "tl-x");
-  value = gs_value_new_double (info->area->x);
-  gsb_scanner_option_set_value (sid, oid, value, error);
-  g_free (value);
-
-  oid = gsb_scanner_get_option_id (sid, "tl-y");
-  value = gs_value_new_double (info->area->y);
-  gsb_scanner_option_set_value (sid, oid, value, error);
-  g_free (value);
-
-  oid = gsb_scanner_get_option_id (sid, "br-x");
-  value = gs_value_new_double (info->area->x + info->area->width);
-  gsb_scanner_option_set_value (sid, oid, value, error);
-  g_free (value);
-
-  oid = gsb_scanner_get_option_id (sid, "br-y");
-  value = gs_value_new_double (info->area->y + info->area->height);
-  gsb_scanner_option_set_value (sid, oid, value, error);
-  g_free (value);
-
-  oid = gsb_scanner_get_option_id (sid, "source");
-  if (oid) {
-    value = gs_value_new_string (g_hash_table_lookup (sid->sources, g_memdup (&(info->source), sizeof (GnomeScannerSource))));
-    gsb_scanner_option_set_value (sid, oid, value, error);
-    g_free (value);
-  }
-#ifdef DEBUG
-  else {
-    g_debug ("Not setting inexistant 'source' option");
-  }
+  gsb_scan_apply_option (info, sid, "tl-x", (GSBOptionValueFunc) gsb_scan_apply_option_value_area, error);
+  gsb_scan_apply_option (info, sid, "tl-y", (GSBOptionValueFunc) gsb_scan_apply_option_value_area, error);
+  gsb_scan_apply_option (info, sid, "br-x", (GSBOptionValueFunc) gsb_scan_apply_option_value_area, error);
+  gsb_scan_apply_option (info, sid, "br-y", (GSBOptionValueFunc) gsb_scan_apply_option_value_area, error);
+  gsb_scan_apply_option (info, sid, "preview", (GSBOptionValueFunc) gsb_scan_apply_option_value_preview, error);
+  gsb_scan_apply_option (info, sid, "source", (GSBOptionValueFunc) gsb_scan_apply_option_value_source, error);
+  gsb_scan_apply_option (info, sid, "mode", (GSBOptionValueFunc) gsb_scan_apply_option_value_colorspace, error);
+  gsb_scan_apply_option (info, sid, "depth", (GSBOptionValueFunc) gsb_scan_apply_option_value_depth, error);
+#if	!ENABLE_PROBE
+  gsb_scan_apply_test_option (sid, "test-picture", gs_value_new_string ("Color pattern"), error);
+  gsb_scan_apply_test_option (sid, "three-pass", gs_value_new_boolean (TRUE), error);
+  gsb_scan_apply_test_option (sid, "three-pass-order", gs_value_new_string ("GBR"), error);
 #endif
-
-#undef	gsb_scanner_get_option_id
 }
 
 /*
@@ -1296,19 +1502,176 @@ gsb_scan_forecast (GnomeScanContextInfo *info,
 
   sid = gnome_scanner_get_id (info->scanner);
   status = sane_get_parameters (sid->handle, &parameters);
-
   if (!(*error = gsb_get_error (status))) {
     forecast = g_new0 (GnomeScanForecast, 1);
     forecast->geometry.width = (gdouble) parameters.pixels_per_line;
     forecast->geometry.height = (gdouble) parameters.lines;
-    forecast->size = parameters.lines * parameters.bytes_per_line;
+    forecast->size = parameters.lines * parameters.bytes_per_line * (parameters.format > SANE_FRAME_RGB ? 3 : 1);
+    forecast->depth = parameters.depth;
     forecast->rowstride = parameters.bytes_per_line;
+    g_debug ("Forecast %fx%f image of %iB", forecast->geometry.width, forecast->geometry.height, forecast->size);
   }
 
   return forecast;
 }
 
 
+/* ACQUISITION */
+
+/* acquire one image */
+GnomeScanResult*
+gsb_acquire_one (GnomeScanBackend *backend,
+		 GnomeScannerId *sid,
+		 GnomeScanContextInfo *info,
+		 GError **error)
+{
+  SANE_Status status;
+  SANE_Parameters parameters;
+  GnomeScannerOption *option;
+  GValue *value;
+  GnomeScanForecast *fc;
+  GnomeScanResult* result;
+  guchar *buffer = NULL, *pixels = NULL;
+  gint frame_count, count, len;
+  gsize size, chunk_size;
+  gboolean first_frame = TRUE;
+
+  result = NULL;
+  count = 0;
+
+  do {				/* Loop each frame */
+    status =  sane_start (sid->handle);
+    if (*error = gsb_get_error (status)) {
+      /* we should handle properly INVAL and other status like that */
+      g_debug ("Failed to start scan from %s: %s", sid->descriptor->name, (*error)->message);
+    }
+    else {
+
+      /* We must forcast after te first sane_start ... */
+      if (first_frame) {
+	fc = gsb_scan_forecast (info, error);
+	if (!*error) {
+	  size = (fc->geometry.width * fc->geometry.height * 3 * 8) / 8;
+	  g_debug ("Final pixbuf size is %iB", size);
+	  pixels = g_try_malloc0 (size);
+	  chunk_size = 32768;
+	  buffer = g_malloc0 (chunk_size);
+	}
+	else {
+	  return NULL;
+	}
+	first_frame = FALSE;
+      }	
+      sane_get_parameters (sid->handle, &parameters);
+      frame_count = 0;
+
+      do {			/* Loop each chunk */
+	/* Real frame acquisition. */
+	status = sane_read (sid->handle, buffer, chunk_size, &len);
+
+	if (status == SANE_STATUS_GOOD) {
+	  gsb_convert_data_to_rgb (parameters.format, fc->depth, buffer, pixels, frame_count, len);
+	  frame_count+= len;
+	  count+= len;
+	  g_debug ("Received %iB; (%i of %iB = %f%%)", len, count, size, 100. * (gdouble) count / (gdouble) size);
+	  g_signal_emit_by_name (backend, "data-received", count);
+	}
+      } while (status == SANE_STATUS_GOOD && count <= fc->size);
+
+      if (status != SANE_STATUS_CANCELLED) {
+	/* other errors are jammed, no docs, cover open, etc. */
+	*error = gsb_get_error (status);
+      }
+    }
+  } while (!parameters.last_frame
+	   && parameters.format != SANE_FRAME_RGB && parameters.format != SANE_FRAME_GRAY);
+
+  if (status == SANE_STATUS_EOF || status == SANE_STATUS_GOOD) {
+    /* Store result */
+    result = g_new0 (GnomeScanResult, 1);
+
+    g_debug ("Retrieving final resolution");
+    /* we retrieve the actual resolution */
+      
+    option = g_hash_table_lookup (sid->hidden, "resolution");
+    if (option) {
+      value = gsb_scanner_option_get_value (sid, gnome_scanner_option_get_id (option), error);
+      switch (G_VALUE_TYPE (value)) {
+      case G_TYPE_INT:
+        result->resolution = (gdouble) g_value_get_int (value);
+	break;
+      case G_TYPE_DOUBLE:
+        result->resolution = g_value_get_double (value);
+	break;
+      }
+    }
+    /* or use the asked one. */
+    else {
+	result->resolution = info->resolution;
+    }
+
+  g_debug ("Final resolution is %f",
+	   result->resolution);
+
+  result->preview = info->preview;
+  result->image = gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB,
+					    FALSE, /* has_alpha */
+					    8, /* bit_per_sample */
+					    fc->geometry.width, fc->geometry.height,
+					    fc->geometry.width * 3, /* rowstride */
+					    (GdkPixbufDestroyNotify) g_free, NULL);
+  }
+    
+  if (buffer) {
+    g_free (buffer);
+
+    /*
+     * Some backend NEED to cancel an acquisition even if terminated
+     * (e.g. hpaio).
+     */
+    sane_cancel (sid->handle);
+  }
+
+  return result;
+}
+
+void
+gsb_convert_data_to_rgb (SANE_Frame format,
+			 gint depth,
+			 guchar *data,
+			 guchar *pixels,
+			 gint offset,
+			 gint length)
+{
+  int i;
+
+  switch (format) {
+  case SANE_FRAME_RGB:
+    memcpy (pixels+offset, data, length);
+    break;
+  case SANE_FRAME_GRAY:
+    offset*=3;
+    for (i = 0 ; i < length ; i++) {
+      *(pixels+offset+(3*i)+0) = data[i]; /* RED */
+      *(pixels+offset+(3*i)+1) = data[i]; /* GREEN */
+      *(pixels+offset+(3*i)+2) = data[i]; /* BLUE */
+    }
+    break;
+  case SANE_FRAME_RED:
+  case SANE_FRAME_GREEN:
+  case SANE_FRAME_BLUE:
+    offset*=3;
+    for (i = 0; i < length ; i++) {
+      /* format = 2, 3 or 4. So format-2 = 0, 1 or 2 */
+      *(pixels+offset+(3*i)+(format-2)) = data[i];
+    }
+    break;
+  default:
+    g_debug ("Frame format (%i) is not supported", format);
+    break;
+  }
+}
+
 /* MISCELLANEOUS FUNCTIONS */
 
 /*
