@@ -48,8 +48,7 @@ enum {
   PROP_BACKEND,
   PROP_SCANNERS,
   PROP_SCANNER,
-  PROP_XRESOLUTION,
-  PROP_YRESOLUTION,
+  PROP_RESOLUTION,
   PROP_AREA,
   PROP_ROTATION
 };
@@ -146,25 +145,15 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
 							G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
-				   PROP_XRESOLUTION,
-				   g_param_spec_int ("x-resolution",
-						     "X resolution",
+				   PROP_RESOLUTION,
+				   g_param_spec_int ("resolution",
+						     "Resolution",
 						     "The final image X resolution.",
 						     0,
 						     G_MAXINT,
 						     100,
 						     G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class,
-				   PROP_YRESOLUTION,
-				   g_param_spec_int ("y-resolution",
-						     "Y resolution",
-						     "The final image y resolution.",
-						     0,
-						     G_MAXINT,
-						     100,
-						     G_PARAM_READWRITE));
-
   g_object_class_install_property (gobject_class,
 				   PROP_AREA,
 				   g_param_spec_pointer ("area",
@@ -482,11 +471,8 @@ gnome_scan_context_set_property (GObject *obj,
   case PROP_SCANNER:
     priv->info->scanner = g_value_get_pointer (value);
     break;
-  case PROP_XRESOLUTION:
-    priv->info->xresolution = g_value_get_double (value);
-    break;
-  case PROP_YRESOLUTION:
-    priv->info->yresolution = g_value_get_double (value);
+  case PROP_RESOLUTION:
+    priv->info->resolution = g_value_get_double (value);
     break;
   case PROP_AREA:
     priv->info->area = g_value_get_pointer (value);
@@ -515,11 +501,8 @@ gnome_scan_context_get_property (GObject *obj,
   case PROP_SCANNER:
     g_value_set_pointer (value, priv->info->scanner);
     break;
-  case PROP_XRESOLUTION:
-    g_value_set_double (value, priv->info->xresolution);
-    break;
-  case PROP_YRESOLUTION:
-    g_value_set_double (value, priv->info->yresolution);
+  case PROP_RESOLUTION:
+    g_value_set_double (value, priv->info->resolution);
     break;
   case PROP_AREA:
     g_value_set_pointer (value, priv->info->area);
@@ -692,7 +675,7 @@ gnome_scan_context_select_scanner (GnomeScanContext *context,
 }
 
 /**
- * gnome_scan_context_set_resolutions:
+ * gnome_scan_context_set_resolution:
  * @context: a #GnomeScanContext
  * @resolution: the new resolution
  * 
@@ -700,31 +683,26 @@ gnome_scan_context_select_scanner (GnomeScanContext *context,
  * allowed value.
  */
 void
-gnome_scan_context_set_resolutions (GnomeScanContext *context,
-				   gdouble xresolution,
-				    gdouble yresolution)
+gnome_scan_context_set_resolution (GnomeScanContext *context,
+				   gdouble resolution)
 {
   g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
-  GET_PRIVATE (context)->info->xresolution = xresolution;
-  GET_PRIVATE (context)->info->yresolution = yresolution;
-  g_signal_emit_by_name (context, "changed", "x-resolution");
-  g_signal_emit_by_name (context, "changed", "y-resolution");
+  GET_PRIVATE (context)->info->resolution = resolution;
+  g_signal_emit_by_name (context, "changed", "resolution");
 }
 
 /**
- * gnome_scan_context_get_resolutions:
+ * gnome_scan_context_get_resolution:
  * @context: a #GnomeScanContext
  * 
  * Retreive the current resolution the image will be scan on.
  */
 void
-gnome_scan_context_get_resolutions (GnomeScanContext *context,
-				   gdouble *xresolution,
-				   gdouble *yresolution)
+gnome_scan_context_get_resolution (GnomeScanContext *context,
+				   gdouble *resolution)
 {
   g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
-  *xresolution = GET_PRIVATE (context)->info->xresolution;
-  *yresolution = GET_PRIVATE (context)->info->yresolution;
+  *resolution = GET_PRIVATE (context)->info->resolution;
 }
 
 /**
@@ -917,8 +895,7 @@ gnome_scan_context_acquire_preview (GnomeScanContext *context)
   /* We create a temp ContextInfo suitable for preview. */
   /* resolution */
   info = g_memdup (priv->info, sizeof (GnomeScanContextInfo));
-  info->xresolution = 50;
-  info->yresolution = 50;
+  info->resolution = 50;
   info->preview = TRUE;
 
   /* area */
