@@ -130,12 +130,10 @@ void			gnome_scan_context_select_scanner		(GnomeScanContext *context,
 									 GnomeScanner *scanner);
 
 
-void			gnome_scan_context_get_resolutions 		(GnomeScanContext *context,
-									 gdouble *xresolution,
-									 gdouble *yresolution);
-void			gnome_scan_context_set_resolutions		(GnomeScanContext *context,
-									 gdouble xresolution,
-									 gdouble yresolution);
+void			gnome_scan_context_get_resolution 		(GnomeScanContext *context,
+									 gdouble *resolution);
+void			gnome_scan_context_set_resolution		(GnomeScanContext *context,
+									 gdouble resolution);
 
 GnomeScanArea*		gnome_scan_context_get_area			(GnomeScanContext *context);
 void			gnome_scan_context_set_area			(GnomeScanContext *context,
